dnl Get myuname
AC_DEFUN(AC_VAR_MYUNAME,
[AC_CACHE_CHECK([system configuration],
  myuname,
[
# Leave a white space between first two '(' for ksh. The sub-shell is needed
# on some machines to avoid the error message when uname is not found; e.g.
# old SUN-OS 3.2 would not execute hostname in (uname -a || hostname). Sigh!
myuname=`( (uname -a) 2>/dev/null || hostname) 2>&1`
# Special mention for Xenix, whose 'uname -a' gives us output like this:
#  sysname=XENIX
#  nodename=whatever
#  release=2.3.2 .. etc...
# Therefore, we strip all this variable assignment junk and remove all the
# new lines to keep the myuname variable sane... --Roland McGrath
myuname=`echo $myuname | sed -e 's/^[^=]*=//' | tr '\012' ' '`
])
AC_SUBST(myuname)dnl
])

dnl Check for signal system type
dnl This is not at all complete check yet, but it have to do for the moment.
dnl There is three systems detected, SYSV, 43BSD and POSIX
dnl There is two name collisions not resolved in terms of side-effect, they are
signal (SYSV and 43BSD) and
dnl sigpause (SYSV and 43BSD).

AC_DEFUN(AC_SYS_SIGNAL,
[
   dnl Tell what we are looking for
   AC_MSG_CHECKING(which signal system is being used)

   dnl Make possibly cached check
   AC_CACHE_VAL(ac_cv_sys_signal,
   [
      dnl Default value
      ac_cv_sys_signal=unknown

      dnl Check for SYSV signal (good old unix.... way before things got complex again :)

      dnl Check the SysV signal hook function
      AC_EGREP_HEADER(signal, signal.h, sysv_signal=yes, sysv_signal=no)

      dnl Check the SysV signal hook & mask function
      AC_EGREP_HEADER(sigset, signal.h, sysv_sigset=yes, sysv_sigset=no)

      dnl Check the SysV signal mask helpers
      AC_EGREP_HEADER(sighold, signal.h, sysv_sighold=yes, sysv_sighold=no)
      AC_EGREP_HEADER(sigrelse, signal.h, sysv_sigrelse=yes, sysv_sigrelse=no)
      AC_EGREP_HEADER(sigignore, signal.h, sysv_sigignore=yes, sysv_sigignore=no)
      AC_EGREP_HEADER(sigpause, signal.h, sysv_sigpause=yes, sysv_sigpause=no)

      dnl Summarize Check
      sysv_mask_helpers=no
      if test $sysv_sighold = yes -a $sysv_sigrelse = yes -a $sysv_sigignore = yes -a $sysv_sigpause = yes; then
         sysv_mask_helpers=yes
      fi
      if test $sysv_signal = yes -a $sysv_sigset = yes -a $sysv_mask_helpers = yes; then
         ac_cv_sys_signal=SYSV
      fi

      dnl Check for 4.3BSD signal system

      dnl Check the 4.3BSD signal hook function
      AC_EGREP_HEADER(signal, signal.h, _43bsd_signal=yes, _43bsd_signal=no)

      dnl Check the 4.3BSD signal hook function
      AC_EGREP_HEADER(sigvec, signal.h, _43bsd_sigvec=yes, _43bsd_sigvec=no)

      dnl Check the 4.3BSD signal pause
      AC_EGREP_HEADER(sigpause, signal.h, _43bsd_sigpause=yes, _43bsd_sigpause=no)

      dnl Check the 4.3BSD signal masker
      AC_EGREP_HEADER(sigsetmask, signal.h, _43bsd_sigsetmask=yes, _43bsd_sigsetmask=no)
      AC_EGREP_HEADER(sigblock, signal.h, _43bsd_sigblock=yes, _43bsd_sigblock=no)

      dnl Check the 4.3BSD signal mask helpers
      AC_EGREP_HEADER(sigmask, signal.h, _43bsd_sigmask=yes, _43bsd_sigmask=no)

      dnl Check the 4.3BSD unique features
      AC_EGREP_HEADER(sigstack, signal.h, _43bsd_sigstack=yes, _43bsd_sigstack=no)
      AC_EGREP_HEADER(sigreturn, signal.h, _43bsd_sigreturn=yes, _43bsd_sigreturn=no)

      dnl Summarize Check
      _43bsd_maskers=no
      if test $_43bsd_sigsetmask = yes -a $_43bsd_sigblock = yes -a $_43bsd_sigmask = yes -a $_43bsd_sigstack = yes -a $_43bsd_sigreturn = yes; then
         _43bsd_maskers=yes
      fi
      if test $_43bsd_signal = yes -a $_43bsd_sigvec = yes -a $_43bsd_sigpause = yes -a _43bsd_maskers ; then
         ac_cv_sys_signal=43BSD
      fi

      dnl Check for POSIX signal system

      dnl Check the POSIX signal hook function
      AC_EGREP_HEADER(sigaction, signal.h, posix_sigaction=yes, posix_sigaction=no)

      dnl Check the POSIX signal pause
      AC_EGREP_HEADER(sigsuspend, signal.h, posix_sigsuspend=yes, posix_sigsuspend=no)

      dnl Check the POSIX signal masker
      AC_EGREP_HEADER(sigprocmask, signal.h, posix_sigprocmask=yes, posix_sigprocmask=no)

      dnl Check the POSIX signal mask helpers
      AC_EGREP_HEADER(sigaddset, signal.h, posix_sigaddset=yes, posix_sigaddset=no)
      AC_EGREP_HEADER(sigdelset, signal.h, posix_sigdelset=yes, posix_sigdelset=no)
      AC_EGREP_HEADER(sigemptyset, signal.h, posix_sigemptyset=yes, posix_sigemptyset=no)
      AC_EGREP_HEADER(sigfillset, signal.h, posix_sigfillset=yes, posix_sigfillset=no)
      AC_EGREP_HEADER(sigismember, signal.h, posix_sigismember=yes, posix_sigismember=no)

      dnl Check the POSIX signal unique feature
      AC_EGREP_HEADER(sigpending, signal.h, posix_sigpending=yes, posix_sigpending=no)

      dnl Summarize check
      if test $posix_sigaction = yes -a $posix_sigpending = yes -a $posix_sigprocmask = yes -a $posix_sigsuspend = yes -a $posix_sigaddset = yes -a $posix_sigdelset = yes -a $posix_sigemptyset = yes -a $posix_sigismember = yes; then
         ac_cv_sys_signal=POSIX
      fi
   ])

   dnl Define separate flags
   if test $ac_cv_sys_signal = "43BSD"; then
      AC_DEFINE(HAVE_SIGNAL_43BSD)
   else
      if test $ac_cv_sys_signal = "POSIX"; then
         AC_DEFINE(HAVE_SIGNAL_POSIX)
      else
         if test $ac_cv_sys_signal = "SYSV"; then
            AC_DEFINE(HAVE_SIGNAL_SYSV)
         fi
      fi
   fi

   dnl Define common flag
   AC_DEFINE(HAVE_SIGNAL, $ac_cv_sys_signal)

   dnl Show result
   AC_MSG_RESULT($ac_cv_sys_signal)
])

dnl AC_KERNEL_VAR(VARIABLE, VAR-TO-CHECK-FOR)
AC_DEFUN(AC_KERNEL_VAR,
[AC_CACHE_VAL(ac_cv_kernel_var_$1,
[if test -n "[$]$1"; then
  ac_cv_prog_$1="[$]$1" # Let the user override the test.
else
   if test ! -f /tmp/kernel.nm; then
      nm $ac_cv_path_kernel > /tmp/kernel.nm
   fi
   if egrep "^$2 " /tmp/kernel.nm >/dev/null 2>&1; then
      ac_cv_kernel_var_$1="$2"
   else
      if egrep " $2\$" /tmp/kernel.nm >/dev/null 2>&1; then
         ac_cv_kernel_var_$1="$2"
      fi
   fi
fi
])dnl
$1="$ac_cv_kernel_var_$1"
AC_SUBST($1)dnl
])

dnl AC_KERNEL_VARS(VARIABLE, VARS-TO-CHECK-FOR [, VALUE-IF-NOT-FOUND])
dnl example: AC_KERNEL_VARS(KERNEL_TCP_TTL, _tcp_ttl, tcp_ttl)
AC_DEFUN(AC_KERNEL_VARS,
[AC_MSG_CHECKING([for $1])
for ac_var in $2
do
AC_KERNEL_VAR($1, [$]ac_var)
test -n "[$]$1" && break
done
ifelse([$3], , , [test -n "[$]$1" || $1="$3"])
if test -n "[$]$1"; then
  AC_MSG_RESULT([$]$1)
else
  AC_MSG_RESULT(no)
fi
])
